<?php
namespace plugins\file_manager\services;

use plugins\file_manager\library\exceptions\DirectoryNotReadableException;
use plugins\file_manager\library\exceptions\FileNotFoundException;
/**
 * FileSystem
 * @author Jérôme Brilland
 * @version 2.2
 * @copyright (c) 2021-2023, Jérôme Brilland
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License
 */
class FileSystem {

    protected $listener;

    public function __construct( $listener ) {
        $this->listener = $listener;
    }

    public function readDirectory( $path ) {
        if ( is_dir( $path ) ) {
            if (is_readable( $path ) ) {
                if ( $handle = opendir( $path ) ) {
                    $finfo = finfo_open( FILEINFO_MIME_TYPE );
                    $files = [];
                    while ( ( $file = readdir( $handle ) ) !== false ) {
                        if ( ( $file != '.' ) && ( $file != '..' ) ) {
                            $filename = $path . DS . $file;
                            $files[] = [
                                'name' => $file,
                                'type' => filetype( $filename ),
                                'mime_type' => finfo_file( $finfo, $filename ),
                                'size' => filesize( $filename ),
                                'permissions' => substr( sprintf( '%o', fileperms( $filename ) ), -4 ),
                                'modified_at' => date( 'Y-m-d', filemtime( $filename ) )
                            ];
                        }
                    }
                    closedir( $handle );
                    return $files;
                }
            } else {
                throw new DirectoryNotReadableException( sprintf( 'The directory %s is not readable.', $path ) );
            }

        } else {
            throw new FileNotFoundException( sprintf( 'The directory %s does not exist.', $path ) );
        }
    }

    public function createFile( $filename ) {
        ( new CreateFile( $filename ) )->perform( $this->listener );
    }

    public function renameFile( $oldname, $newname ) {
        ( new RenameFile( $oldname, $newname ) )->perform( $this->listener );
    }

    public function deleteFile( $filename ) {
        ( new DeleteFile( $filename ) )->perform( $this->listener );
    }

    public function createDirectory( $path ) {
        ( new CreateDirectory( $path ) )->perform( $this->listener );
    }

    public function deleteDirectory( $path ) {
        ( new DeleteDirectory( $path ) )->perform( $this->listener );

    }

    public function changePermissions( $filename, $permissions ) {
        ( new ChangePermissions( $filename, $permissions ) )->perform( $this->listener );
    }

    public function readFile( $filename ) {
        if ( is_file( $filename ) && is_readable( $filename ) ) {
            $finfo = finfo_open( FILEINFO_MIME_TYPE );
            return [
                'name' => basename( $filename ),
                'mime_type' => finfo_file( $finfo, $filename ),
                'extension' => strtolower( substr( strrchr( $filename, '.' ), 1 ) ),
                'content' => file_get_contents( $filename )
            ];
        } else {
            throw new \RuntimeException( sprintf( 'The file %s does not exist or is not readable.', $filename ) );
        }
    }

    public function writeFile( $filename, $content ) {
        ( new WriteFile( $filename, $content ) )->perform( $this->listener );
    }

    public function uploadFile( $filename, $path ) {
        ( new UploadFile( $filename, $path ) )->perform( $this->listener );
    }
}
